@@ -16,24 +16,24 @@
 ;;; You should have received a copy of the GNU Lesser General Public
 ;;; License along with 8sync.  If not, see <http://www.gnu.org/licenses/>.
 
-(use-modules (8sync systems actors)
+(use-modules (8sync actors)
              (oop goops))
 
-(define-simple-actor <emo>
-  (greet-proog
-   (lambda (actor message target)
-     (display "emo> What's next, Proog?\n")
-     (<- actor target 'greet-emo))))
+(define-actor <emo> (<actor>)
+  ((greet-proog
+    (lambda (actor message target)
+      (display "emo> What's next, Proog?\n")
+      (<- target 'greet-emo)))))
 
-(define-simple-actor <proog>
-  (greet-emo
-   (lambda (actor message)
-     (display "proog> Listen, Emo!  Listen to the sounds of the machine!\n"))))
+(define-actor <proog> (<actor>)
+  ((greet-emo
+    (lambda (actor message)
+      (display "proog> Listen, Emo!  Listen to the sounds of the machine!\n")))))
 
 (define hive (make-hive))
-(define our-emo (hive-create-actor hive <emo>))
-(define our-proog (hive-create-actor hive <proog>))
+(define our-emo (bootstrap-actor hive <emo>))
+(define our-proog (bootstrap-actor hive <proog>))
 (define (main . args)
-  (ez-run-hive hive
-               (list (bootstrap-message hive our-emo 'greet-proog
-                                        our-proog))))
+  (run-hive hive
+            (list (bootstrap-message hive our-emo 'greet-proog
+                                     our-proog))))
